@@ -193,7 +193,7 @@ class FilenameCollectionThread(threading.Thread):
 
         if new_search_dir != self.current_search_dir:
             with self.state_lock:
-                _logger.debug("Switching search directory from {} to {}.".format(self.current_search_dir, new_search_dir))
+                _logger.debug("Switching search directory from {} to {}".format(self.current_search_dir, new_search_dir))
                 self.search_dir_queue.put(new_search_dir)
 
     def get_current_filenames(self):
@@ -320,7 +320,7 @@ class SearchThread(threading.Thread):
             # so... add on an incremental search!
             with self.state_lock:
                 new_files = current_filenames.candidates.difference(self.candidate_fns)
-                _logger.debug("Adding {:d} more files to current search for input_str '{}' in directory {}.".format(len(new_files), input_str, current_filenames.current_search_dir))
+                _logger.debug("Adding {:d} more files to current search for input_str '{}' in directory {}".format(len(new_files), input_str, current_filenames.current_search_dir))
                 self.input_queue.put(self.IncrementalInput(
                     new_candidate_fns=new_files,
                     candidate_computation_complete=current_filenames.candidate_computation_complete
@@ -412,8 +412,9 @@ class SearchThread(threading.Thread):
             # fuzzy matching: for input string abc, find a*b*c substrings (consuming as few characters as possible in between)
             # guard against user input that may be construed as a regex
             regex_str = "(.*?)".join( re.escape(ch) for ch in lowered )
-            # prepend (?:.*) to push off the matching as much as possible
-            regex = re.compile("(?:.*)" + regex_str, re.IGNORECASE | re.DOTALL)
+            filter_regex = re.compile(regex_str, re.IGNORECASE | re.DOTALL)
+            # prepend (?:.*) to push off the matching as much as possible (more expensive but more accurate)
+            ranking_regex = re.compile("(?:.*)" + regex_str, re.IGNORECASE | re.DOTALL)
 
             def get_match_tuples_it():
                 def nonempty_groups(match):
@@ -423,14 +424,15 @@ class SearchThread(threading.Thread):
                     if self._interrupted():
                         raise ComputationInterruptedException("Searching interrupted!")
 
-                    match = regex.search(fn)
-                    if match is not None:
-                        negs = nonempty_groups(match)
+                    filter_match = filter_regex.search(fn)
+                    if filter_match is not None:
+                        ranking_match = ranking_regex.search(fn)
+                        negs = nonempty_groups(ranking_match)
                         yield self.MatchTuple(
-                                string=match.string,
+                                string=fn,
                                 num_nonempty_groups = len(negs),
                                 total_group_length=len("".join(negs)),
-                                num_dirs_in_path=get_num_dirs_in_path(match.string)
+                                num_dirs_in_path=get_num_dirs_in_path(fn)
                                 )
             return list(get_match_tuples_it())
 
@@ -473,11 +475,19 @@ def select_filename(screen, fn_collection_thread, input_str):
 
     search_status = SearchStatus()
 
+    def get_display_uuid(input_str, curr_fns, eligible_fns):
+        """ Returns a unique id to represent what we're currently displaying on the screen.  Useful for us to block if we're not showing anything new. """
+        return hash("".join(map(str,[
+            input_str,
+            curr_fns.candidate_computation_complete, curr_fns.current_search_dir, len(curr_fns.candidates),
+            eligible_fns.search_complete, len(eligible_fns.eligible) ])))
+
     def ensure_threads_alive(*threads):
         for th in threads:
             if not th.is_alive():
                 raise Exception("{} died with traceback:\n{}".format(th, th.get_traceback()))
 
+    prev_display_uuid = None
     while True:
         ensure_threads_alive(fn_collection_thread, search_thread)
 
@@ -535,10 +545,14 @@ def select_filename(screen, fn_collection_thread, input_str):
         # put the cursor at the end of the string
         input_x = min(len(input_str), max_width - 1)
 
-        # getch is nonblocking; try in 20ms increments for up to 120ms before redrawing screen
+        # getch is nonblocking; try in 20ms increments for up to 120ms before redrawing screen (60s if we know the screen won't change without input)
+        new_display_uuid = get_display_uuid(input_str, curr_fns, eligible_fns)
+        getch_time = 60 if new_display_uuid == prev_display_uuid and curr_fns.candidate_computation_complete and eligible_fns.search_complete else 0.120
+        prev_display_uuid = new_display_uuid
+
         start_getch = time.time()
         raw_key = -1
-        while (time.time() - start_getch) < 0.120:
+        while (time.time() - start_getch) < getch_time:
             raw_key = screen.getch(INPUT_Y, input_x)
             if raw_key != -1: break
             time.sleep(0.020)
